#
# CmbToplevelChooser
#
# Copyright (C) 2021-2023  Juan Pablo Ugarte
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation;
# version 2.1 of the License.
#
# library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors:
#   Juan Pablo Ugarte <juanpablougarte@gmail.com>
#

from gi.repository import GObject, Gtk
from cambalache import CmbObject, CmbUI


class CmbToplevelChooser(Gtk.DropDown):
    __gtype_name__ = "CmbToplevelChooser"

    object = GObject.Property(type=CmbUI, flags=GObject.ParamFlags.READWRITE)
    derivable_only = GObject.Property(type=bool, default=False, flags=GObject.ParamFlags.READWRITE)

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

        # TODO: Remove me once we depend on Gtk 4.16
        if not hasattr(self.props, "autoselect"):
            self.props.sensitive = False
            self.props.tooltip_text = "This widget requires Gtk >= 4.16"
            return

        self.props.autoselect = False
        self.props.can_unselect = True

        self.__update_model()

        self.__expression = Gtk.PropertyExpression.new(CmbObject, None, "display-name-type")
        self.props.expression = self.__expression

        self.connect("notify::object", self.__on_object_notify)
        self.connect("notify::selected-item", self.__on_selected_item_notify)

    def __update_model(self):
        if self.object is None:
            self.props.model = None
            return

        if self.derivable_only:
            model = Gtk.FilterListModel(
                model=self.object,
                filter=Gtk.CustomFilter.new(lambda i, d: i.info.derivable, None)
            )
            self.props.model = model
        else:
            self.props.model = self.object

    def __on_object_notify(self, obj, pspec):
        self.props.model = None
        self.__update_model()

    def __on_selected_item_notify(self, obj, pspec):
        self.notify("cmb-value")

    @GObject.Property(type=int)
    def cmb_value(self):
        item = self.get_selected_item()
        if item is None:
            return 0

        return item.object_id

    @cmb_value.setter
    def _set_cmb_value(self, value):
        if self.object is None:
            return

        item = self.object.project.get_object_by_id(self.object.ui_id, value)

        if item:
            found, position = self.props.model.find(item)
            if found:
                self.set_selected(position)
        else:
            self.set_selected(Gtk.INVALID_LIST_POSITION)
